"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseAsCustomType = exports.parseAsNull = exports.parseAsNumber = exports.parseAsString = exports.parseAsBoolean = void 0;
const option_1 = require("../option");
const parser_1 = require("./parser");
const util_1 = require("./util");
const parseAsNull = (t) => (0, parser_1.createPrimitiveParser)({
    t,
    expectedType: 'null',
    isMatchingType: () => t === null,
});
exports.parseAsNull = parseAsNull;
const parseAsString = (t) => {
    const isMatchingType = () => (0, util_1.isEqual)('string', typeof t);
    const orElseGetEmptyString = () => (!isMatchingType() ? '' : t);
    return {
        ...(0, parser_1.createPrimitiveParser)({
            t,
            isMatchingType,
            expectedType: 'string',
        }),
        orElseGetEmptyString,
        exactlyAs: (s) => ({
            ...(0, option_1.createExact)({
                t,
                e: s,
                expectedType: 'string',
            }),
            orElseGetEmptyString,
        }),
    };
};
exports.parseAsString = parseAsString;
const parseAsBoolean = (t) => {
    const expectedType = 'boolean';
    const isMatchingType = () => (0, util_1.isEqual)(expectedType, typeof t);
    const orElseGet = (b) => (!isMatchingType() ? b : t);
    const orElseGetTrue = () => orElseGet(true);
    const orElseGetFalse = () => orElseGet(false);
    return {
        ...(0, parser_1.createPrimitiveParser)({
            t,
            isMatchingType,
            expectedType: 'boolean',
        }),
        orElseGetTrue,
        orElseGetFalse,
        exactlyAs: (b) => ({
            ...(0, option_1.createExact)({
                t,
                e: b,
                expectedType: 'boolean',
            }),
            orElseGetTrue,
            orElseGetFalse,
        }),
    };
};
exports.parseAsBoolean = parseAsBoolean;
const parseAsNumber = (t) => {
    const expectedType = 'number';
    const isMatchingType = () => (0, util_1.isEqual)(expectedType, typeof t);
    return {
        ...(0, parser_1.createPrimitiveParser)({
            t,
            expectedType,
            isMatchingType,
        }),
        inRangeOf: (min, max) => {
            if (min > max || max < min) {
                throw new Error(`min must be less than max, max must also be greater than min. Min: ${min}, Max: ${max}`);
            }
            const options = (0, option_1.createOptions)({
                t: () => t,
                expectedType,
                isMatchingType: () => {
                    if (!isMatchingType()) {
                        return false;
                    }
                    const n = t;
                    return n >= min && n <= max;
                },
            });
            return options;
        },
        exactlyAs: (n) => ({
            ...(0, option_1.createExact)({
                t,
                e: n,
                expectedType,
            }),
        }),
    };
};
exports.parseAsNumber = parseAsNumber;
/**
 * Although it can be used to parse anything, this is recommended to be used to parse as a specified non-object type
 * @returns the Options object
 */
const parseAsCustomType = (t, isParseable) => (0, option_1.createOptions)({
    t: () => t,
    isMatchingType: () => isParseable(t),
    expectedType: (0, util_1.getPossibleSelfDefinedType)(t),
});
exports.parseAsCustomType = parseAsCustomType;
//# sourceMappingURL=primitive.js.map